from typing import Any, Dict

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ImageDigest(BaseK8sContainerCheck):
    def __init__(self) -> None:
        """
        The image specification should use a digest instead of a tag to make sure the container always uses the same
        version of the image.
        https://kubernetes.io/docs/concepts/configuration/overview/#container-images

        An admission controller could be used to enforce the use of image digest
        """
        name = "Image should use digest"
        id = "CKV_K8S_43"
        # Location: container .image
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["image"]
        if "image" in conf:

            # The @ indicates there is a digest. It's technically possible to use a tag as well, but it doesn't make
            # a difference. So, this @ is all we need to pass the check.
            image_conf = conf["image"]
            if isinstance(image_conf, str):
                has_digest = "@" in image_conf
                return CheckResult.PASSED if has_digest else CheckResult.FAILED
        return CheckResult.FAILED


check = ImageDigest()



from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ECRImmutableTags(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure ECR Image Tags are immutable"
        id = "CKV_AWS_51"
        supported_resources = ["AWS::ECR::Repository"]
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "Properties/ImageTagMutability"

    def get_expected_value(self):
        return "IMMUTABLE"

check = ECRImmutableTags()



from typing import List

from checkov.common.graph.db_connectors.db_connector import DBConnector
from checkov.common.graph.graph_manager import GraphManager
from checkov.kubernetes.graph_builder.local_graph import KubernetesLocalGraph
from checkov.kubernetes.kubernetes_utils import get_folder_definitions


class KubernetesGraphManager(GraphManager):
    def __init__(self, db_connector: DBConnector, source: str = "Kubernetes") -> None:
        super().__init__(db_connector=db_connector, parser=None, source=source)

    def build_graph_from_source_directory(self, source_dir, render_variables=True,
                                          local_graph_classType : [KubernetesLocalGraph] = KubernetesLocalGraph,
                                          parsing_errors=None, download_external_modules=False,
                                          excluded_paths: List[str] = None):
        definitions, definitions_raw = get_folder_definitions(source_dir, excluded_paths)
        local_graph = self.build_graph_from_definitions(definitions, False)
        return local_graph, definitions

    def build_graph_from_definitions(self, definitions, render_variables=True):
        local_graph = KubernetesLocalGraph(definitions)
        local_graph.build_graph(False)
        return local_graph



import json
import logging
from typing import List, Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver

logger = logging.getLogger(__name__)


class ContainsAttributeSolver(BaseAttributeSolver):
    operator = Operators.CONTAINS

    def __init__(self, resource_types: List[str], attribute: Optional[str], value: Any) -> None:
        super().__init__(resource_types=resource_types, attribute=attribute, value=value)

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        att = vertex.get(attribute, "{}")
        if isinstance(att, str):
            try:
                att = json.loads(att.replace("'", '"'))
            except ValueError:
                pass
        if isinstance(att, dict):
            return self.value in att or any(self.value in val for val in att.values() if type(val) in [str, list, set, dict])
        return self.value in att




import collections

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck


class AliasIsUnique(BaseDockerfileCheck):
    def __init__(self):
        """
        Ensure From Alias are unique for multistage builds.
        """
        name = "Ensure From Alias are unique for multistage builds."
        id = "CKV_DOCKER_11"
        supported_instructions = ["FROM"]
        categories = [CheckCategories.CONVENTION]
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_entity_conf(self, conf):
        alias = []
        for instruction in conf:
            if " as " in instruction["value"]:
                temp = instruction["value"].split()
                alias += [temp[2]]

        if len(alias) == len(set(alias)):
            return CheckResult.PASSED, None
        else:
            return CheckResult.FAILED, conf[0]


check = AliasIsUnique()



from checkov.common.output.report import CheckType
from checkov.common.parsers.yaml.parser import parse
from checkov.common.runners.object_runner import Runner as ObjectRunner


class Runner(ObjectRunner):
    check_type = CheckType.YAML

    def import_registry(self):
        from checkov.yaml_doc.registry import registry
        return registry

    def _parse_file(self, f):
        content = parse(f)
        return content

    def get_start_end_lines(self, end, result_config, start):
        if result_config and isinstance(result_config, list):
            start = result_config[0]['__startline__']
            end = result_config[len(result_config) - 1]['__endline__']
        elif result_config and isinstance(result_config, dict):
            start = result_config['__startline__']
            end = result_config['__endline__']
        return end, start



from update_checker import UpdateChecker


def check_for_update(package, version):
    try:
        checker = UpdateChecker()
        result = checker.check(package, version)
        return result.available_version
    except Exception:  # nosec
        return None



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class InterfaceMgmtProfileNoHTTP(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure plain-text management HTTP is not enabled for an Interface Management Profile"
        id = "CKV_PAN_2"
        supported_resources = ['panos_management_profile']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'http'

    def get_forbidden_values(self):
        return [True]


check = InterfaceMgmtProfileNoHTTP()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AutomationEncrypted(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Automation account variables are encrypted"
        id = "CKV_AZURE_73"
        supported_resources = (
            "azurerm_automation_variable_bool",
            "azurerm_automation_variable_string",
            "azurerm_automation_variable_int",
            "azurerm_automation_variable_datetime",
        )
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "encrypted"


check = AutomationEncrypted()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureDataExplorerDoubleEncryptionEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Azure Data Explorer uses double encryption"
        id = "CKV_AZURE_75"
        supported_resources = ['azurerm_kusto_cluster']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "double_encryption_enabled"

    def get_expected_value(self):
        return True


check = AzureDataExplorerDoubleEncryptionEnabled()



from __future__ import annotations

import re
from pathlib import Path
from typing import cast

from typing_extensions import Literal

from checkov.bicep.checks.param.registry import registry as param_registry
from checkov.bicep.checks.resource.registry import registry as resource_registry
from checkov.bicep.parser import Parser
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import CheckType, Report
from checkov.common.runners.base_runner import BaseRunner
from checkov.common.typing import _CheckResult
from checkov.runner_filter import RunnerFilter


class Runner(BaseRunner):
    check_type = CheckType.BICEP
    bicep_cli = "bicep"

    block_type_registries: dict[Literal["parameters", "resources"], BaseCheckRegistry] = {
        "parameters": param_registry,
        "resources": resource_registry,
    }

    def run(
        self,
        root_folder: str | Path,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter = RunnerFilter(),
        collect_skip_comments: bool = True,
    ) -> Report:
        report = Report(Runner.check_type)

        file_paths: set[Path] = set()
        if root_folder:
            root_path = Path(root_folder)
            file_paths = {file_path for file_path in root_path.rglob("*.bicep")}
        if files:
            for file in files:
                if file.endswith(".bicep"):
                    file_paths.add(Path(file))

        definitions, definitions_raw, parsing_errors = Parser().get_files_definitions(file_paths)

        report.add_parsing_errors(parsing_errors)

        for file_path, definition in definitions.items():
            for block_type, registry in Runner.block_type_registries.items():
                block_type_confs = definition.get(block_type)
                if block_type_confs:
                    for name, conf in block_type_confs.items():
                        results = registry.scan(
                            scanned_file=str(file_path),
                            entity={name: conf},
                            skipped_checks=[],
                            runner_filter=runner_filter,
                        )

                        if results:
                            file_code_lines = definitions_raw[file_path]
                            start_line = cast(int, conf["__start_line__"])  # it is alwasy set for the main block types
                            end_line = cast(int, conf["__end_line__"])  # it is alwasy set for the main block types

                            cleaned_path = self.clean_file_path(file_path)
                            resource_id = f"{conf['type']}.{name}"
                            report.add_resource(f"{cleaned_path}:{resource_id}")

                            suppressions = self.search_for_suppression(
                                code_lines=file_code_lines[start_line - 1 : end_line]
                            )

                            for check, check_result in results.items():
                                if check.id in suppressions.keys():
                                    check_result = suppressions[check.id]
                                elif check.bc_id and check.bc_id in suppressions.keys():
                                    check_result = suppressions[check.bc_id]

                                record = Record(
                                    check_id=check.id,
                                    bc_check_id=check.bc_id,
                                    check_name=check.name,
                                    check_result=check_result,
                                    code_block=file_code_lines[start_line - 1 : end_line],
                                    file_path=str(cleaned_path),
                                    file_line_range=[start_line, end_line],
                                    resource=resource_id,
                                    check_class=check.__class__.__module__,
                                    file_abs_path=str(file_path.absolute()),
                                    evaluations=None,
                                    severity=check.severity,
                                )
                                record.set_guideline(check.guideline)
                                report.add_record(record=record)

        return report

    def search_for_suppression(self, code_lines: list[tuple[int, str]]) -> dict[str, _CheckResult]:
        suppressions = {}

        for _, line in code_lines:
            skip_search = re.search(COMMENT_REGEX, line)
            if skip_search:
                check_result: _CheckResult = {
                    "result": CheckResult.SKIPPED,
                    "suppress_comment": skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided",
                }
                suppressions[skip_search.group(2)] = check_result

        return suppressions

    def clean_file_path(self, file_path: Path) -> Path:
        path_parts = [part for part in file_path.parts if part not in (".", "..")]

        return Path(*path_parts)



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class StorageBlockBackupEnabled(BaseResourceValueCheck):

    def __init__(self):
        name = "Ensure OCI Block Storage Block Volume has backup enabled"
        id = "CKV_OCI_2"
        supported_resources = ['oci_core_volume']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "backup_policy_id"

    def get_expected_value(self):
        return ANY_VALUE


check = StorageBlockBackupEnabled()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class MWAASchedulerLogsEnabled(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure MWAA environment has scheduler logs enabled"
        id = "CKV_AWS_242"
        supported_resources = ["aws_mwaa_environment"]
        categories = [CheckCategories.LOGGING]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "logging_configuration/[0]/scheduler_logs/[0]/enabled"


check = MWAASchedulerLogsEnabled()



#!/usr/bin/env python

import re
from typing import List, Optional, Tuple, Union

from tabulate import tabulate

from checkov.arm.registry import arm_resource_registry, arm_parameter_registry
from checkov.bicep.checks.param.registry import registry as bicep_param_registry
from checkov.bicep.checks.resource.registry import registry as bicep_resource_registry
from checkov.bitbucket.registry import registry as bitbucket_configuration_registry
from checkov.cloudformation.checks.resource.registry import cfn_registry as cfn_registry
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.checks_infra.registry import BaseRegistry as BaseGraphRegistry, get_graph_checks_registry
from checkov.dockerfile.registry import registry as dockerfile_registry
from checkov.github.registry import registry as github_configuration_registry
from checkov.github_actions.checks.job_registry import registry as github_actions_jobs_registry
from checkov.gitlab.registry import registry as gitlab_configuration_registry
from checkov.kubernetes.checks.resource.registry import registry as k8_registry
from checkov.secrets.runner import CHECK_ID_TO_SECRET_TYPE
from checkov.serverless.registry import sls_registry
from checkov.terraform.checks.data.registry import data_registry
from checkov.terraform.checks.module.registry import module_registry
from checkov.terraform.checks.provider.registry import provider_registry
from checkov.terraform.checks.resource.registry import resource_registry
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration


ID_PARTS_PATTERN = re.compile(r'([^_]*)_([^_]*)_(\d+)')


def get_compare_key(c):
    res = []
    for match in ID_PARTS_PATTERN.finditer(c[0]):
        ckv, framework, number = match.groups()
        numeric_value = int(number) if number else 0
        # count number of leading zeros
        same_number_ordering = len(number) - len(number.lstrip('0'))
        res.append((framework, ckv, numeric_value, same_number_ordering, c[2]))
    return res


def print_checks(frameworks: Optional[List[str]] = None, use_bc_ids: bool = False) -> None:
    framework_list = frameworks if frameworks else ["all"]
    printable_checks_list = get_checks(framework_list, use_bc_ids=use_bc_ids)
    print(
        tabulate(printable_checks_list, headers=["Id", "Type", "Entity", "Policy", "IaC"], tablefmt="github",
                 showindex=True))
    print("\n\n---\n\n")


def get_checks(frameworks: Optional[List[str]] = None, use_bc_ids: bool = False) -> \
        List[Tuple[str, str, str, str, str]]:
    framework_list = frameworks if frameworks else ["all"]
    printable_checks_list = []

    def add_from_repository(registry: Union[BaseCheckRegistry, BaseGraphRegistry], checked_type: str, iac: str) -> None:
        nonlocal printable_checks_list
        if isinstance(registry, BaseCheckRegistry):
            for entity, check in registry.all_checks():
                printable_checks_list.append((check.get_output_id(use_bc_ids), checked_type, entity, check.name, iac))
        elif isinstance(registry, BaseGraphRegistry):
            for check in registry.checks:
                for rt in check.resource_types:
                    printable_checks_list.append((check.get_output_id(use_bc_ids), checked_type, rt, check.name, iac))

    if any(x in framework_list for x in ("all", "terraform")):
        add_from_repository(resource_registry, "resource", "Terraform")
        add_from_repository(data_registry, "data", "Terraform")
        add_from_repository(provider_registry, "provider", "Terraform")
        add_from_repository(module_registry, "module", "Terraform")

        graph_registry = get_graph_checks_registry("terraform")
        graph_registry.load_checks()
        add_from_repository(graph_registry, "resource", "Terraform")
    if any(x in framework_list for x in ("all", "cloudformation")):
        add_from_repository(cfn_registry, "resource", "Cloudformation")
    if any(x in framework_list for x in ("all", "kubernetes")):
        add_from_repository(k8_registry, "resource", "Kubernetes")
    if any(x in framework_list for x in ("all", "serverless")):
        add_from_repository(sls_registry, "resource", "serverless")
    if any(x in framework_list for x in ("all", "dockerfile")):
        add_from_repository(dockerfile_registry, "dockerfile", "dockerfile")
    if any(x in framework_list for x in ("all", "github_configuration")):
        add_from_repository(github_configuration_registry, "github_configuration", "github_configuration")
    if any(x in framework_list for x in ("all", "github_actions")):
        add_from_repository(github_actions_jobs_registry, "jobs", "github_actions")
    if any(x in framework_list for x in ("all", "gitlab_configuration")):
        add_from_repository(gitlab_configuration_registry, "gitlab_configuration", "gitlab_configuration")
    if any(x in framework_list for x in ("all", "bitbucket_configuration")):
        add_from_repository(bitbucket_configuration_registry, "bitbucket_configuration", "bitbucket_configuration")
    if any(x in framework_list for x in ("all", "arm")):
        add_from_repository(arm_resource_registry, "resource", "arm")
        add_from_repository(arm_parameter_registry, "parameter", "arm")
    if any(x in framework_list for x in ("all", "bicep")):
        add_from_repository(bicep_param_registry, "parameter", "Bicep")
        add_from_repository(bicep_resource_registry, "resource", "Bicep")
    if any(x in framework_list for x in ("all", "secrets")):
        for check_id, check_type in CHECK_ID_TO_SECRET_TYPE.items():
            if use_bc_ids:
                check_id = metadata_integration.get_bc_id(check_id)
            printable_checks_list.append((check_id, check_type, "secrets", check_type, "secrets"))
    return sorted(printable_checks_list, key=get_compare_key)


if __name__ == '__main__':
    print_checks()



from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SharedHostNetworkNamespace(BaseResourceValueCheck):

    def __init__(self):
        # CIS-1.3 1.7.4
        # CIS-1.5 5.2.4
        name = "Do not admit containers wishing to share the host network namespace"
        id = "CKV_K8S_19"
        supported_resources = ["kubernetes_pod"]
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self) -> str:
        return "spec/[0]/host_network"

    def get_expected_value(self) -> Any:
        return False


check = SharedHostNetworkNamespace()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class ApiServerTlsCertAndKey(BaseK8sContainerCheck):
    def __init__(self) -> None:
        id = "CKV_K8S_100"
        name = "Ensure that the --tls-cert-file and --tls-private-key-file arguments are set as appropriate"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["command"]
        if conf.get("command"):
            if "kube-apiserver" in conf["command"]:
                hasCertCommand = False
                hasKeyCommand = False
                for command in conf["command"]:
                    if command.startswith("--tls-cert-file"):
                        hasCertCommand = True
                    elif command.startswith("--tls-private-key-file"):
                        hasKeyCommand = True
                return CheckResult.PASSED if hasCertCommand and hasKeyCommand else CheckResult.FAILED

        return CheckResult.PASSED


check = ApiServerTlsCertAndKey()



# method 'str.removeprefix()' was added in Python 3.9
def removeprefix(input_str: str, prefix: str) -> str:
    if input_str.startswith(prefix):
        return input_str[len(prefix) :]
    return input_str



import logging
import os
from abc import abstractmethod

from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.runner_filter import RunnerFilter


class Runner(BaseRunner):

    def _load_files(self, files_to_load, definitions, definitions_raw, filename_fn=None):
        files_to_load = [filename_fn(file) if filename_fn else file for file in files_to_load]
        results = parallel_runner.run_function(lambda f: (f, self._parse_file(f)), files_to_load)
        for file, result in results:
            if result:
                (definitions[file], definitions_raw[file]) = result

    @abstractmethod
    def _parse_file(self, f):
        raise Exception("parser should be imported by deriving class")

    def run(self, root_folder=None, external_checks_dir=None, files=None,
            runner_filter=RunnerFilter(), collect_skip_comments=True) -> Report:
        registry = self.import_registry()

        definitions = {}
        definitions_raw = {}

        report = Report(self.check_type)

        if not files and not root_folder:
            logging.debug("No resources to scan.")
            return report

        if not external_checks_dir and self.require_external_checks():
            logging.debug("The runner requires that external checks are defined.")
            return report
        if external_checks_dir:
            for directory in external_checks_dir:
                registry.load_external_checks(directory)

        if files:
            self._load_files(files, definitions, definitions_raw)

        if root_folder:
            for root, d_names, f_names in os.walk(root_folder):
                filter_ignored_paths(root, d_names, runner_filter.excluded_paths)
                filter_ignored_paths(root, f_names, runner_filter.excluded_paths)
                self._load_files(
                    f_names,
                    definitions,
                    definitions_raw,
                    lambda f: os.path.join(root, f)
                )

        for file_path in definitions.keys():
            results = registry.scan(
                file_path, definitions[file_path], [], runner_filter
            )
            for key, result in results.items():
                result_config = result["results_configuration"]
                start = 0
                end = 0
                check = result["check"]
                end, start = self.get_start_end_lines(end, result_config, start)
                record = Record(
                    check_id=check.id,
                    bc_check_id=check.bc_id,
                    check_name=check.name,
                    check_result=result,
                    code_block=definitions_raw[file_path][start:end + 1],
                    file_path=file_path,
                    file_line_range=[start + 1, end + 1],
                    resource=f"{file_path}.{key}",
                    evaluations=None,
                    check_class=check.__class__.__module__,
                    file_abs_path=os.path.abspath(file_path),
                    entity_tags=None,
                    severity=check.severity
                )
                report.add_record(record)

        return report

    @abstractmethod
    def get_start_end_lines(self, end, result_config, start):
        raise Exception("should be handled by derived class")

    @abstractmethod
    def import_registry(self):
        raise Exception("registry should be imported by deriving class")

    def require_external_checks(self):
        return True

    @staticmethod
    def _change_files_path_to_relative(report: Report):
        for record in report.get_all_records():
            record.file_path = record.file_path.replace(os.getcwd(), '')
            record.resource = record.resource.replace(os.getcwd(), '')



from typing import Any, List

from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class CloudFrontTLS12(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Verify CloudFront Distribution Viewer Certificate is using TLS v1.2"
        id = "CKV_AWS_174"
        supported_resources = ["AWS::CloudFront::Distribution"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/DistributionConfig/ViewerCertificate/MinimumProtocolVersion"

    def get_expected_values(self) -> List[str]:
        return ['TLSv1.2_2018', 'TLSv1.2_2019', 'TLSv1.2_2021']

    def get_expected_value(self) -> Any:
        return 'TLSv1.2_2021'


check = CloudFrontTLS12()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ElasticacheReplicationGroupEncryptionAtTransitAuthToken(BaseResourceCheck):
    def __init__(self):
        name = "Ensure all data stored in the Elasticache Replication Group is securely encrypted at transit and has auth token"
        id = "CKV_AWS_31"
        supported_resources = ['AWS::ElastiCache::ReplicationGroup']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for encryption configuration at aws_elasticache_replication_group:
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticache-replicationgroup.html
        :param conf: aws_elasticache_replication_group configuration
        :return: <CheckResult>
        """
        if 'Properties' in conf.keys():
            if 'TransitEncryptionEnabled' in conf['Properties'].keys() and 'AuthToken' in conf['Properties'].keys():
                if conf['Properties']['TransitEncryptionEnabled']:
                    return CheckResult.PASSED
        return CheckResult.FAILED

check = ElasticacheReplicationGroupEncryptionAtTransitAuthToken()



from checkov.terraform.checks.resource.aws.AbsNACLUnrestrictedIngress import AbsNACLUnrestrictedIngress


class NACLUnrestrictedIngress21(AbsNACLUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_229", port=21)


check = NACLUnrestrictedIngress21()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import Any, List


class ECSClusterLoggingEnabled(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure ECS Cluster enables logging of ECS Exec"
        id = "CKV_AWS_223"
        supported_resources = ["aws_ecs_cluster"]
        categories = [CheckCategories.LOGGING]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "configuration/[0]/execute_command_configuration/[0]/logging"

    def get_forbidden_values(self) -> List[Any]:
        return ["NONE"]


check = ECSClusterLoggingEnabled()



import re
from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.provider.base_check import BaseProviderCheck
from checkov.common.models.consts import panos_api_key_pattern


class PanosCredentials(BaseProviderCheck):
    def __init__(self) -> None:
        name = "Ensure no hard coded PAN-OS credentials exist in provider"
        id = "CKV_PAN_1"
        supported_provider = ["panos"]
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_provider=supported_provider)

    def scan_provider_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if self.secret_found(conf, "api_key", panos_api_key_pattern):
            return CheckResult.FAILED
        if conf.get("password"):
            return CheckResult.FAILED
        return CheckResult.PASSED

    @staticmethod
    def secret_found(conf: Dict[str, List[Any]], field: str, pattern: str) -> bool:
        if field in conf.keys():
            value = conf[field][0]
            if not isinstance(value, str) or re.match(pattern, value) is not None:
                return True
        return False


check = PanosCredentials()



import concurrent.futures
import logging
from typing import List, Dict, Any

from networkx import DiGraph

from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
from checkov.common.graph.checks_infra.base_parser import BaseGraphCheckParser
from checkov.common.models.enums import CheckResult
from checkov.runner_filter import RunnerFilter


class BaseRegistry:
    def __init__(self, parser: BaseGraphCheckParser) -> None:
        self.checks: List[BaseGraphCheck] = []
        self.parser = parser

    def load_checks(self):
        raise NotImplementedError

    def run_checks(
        self, graph_connector: DiGraph, runner_filter: RunnerFilter
    ) -> Dict[BaseGraphCheck, List[Dict[str, Any]]]:
        check_results = {}
        checks_to_run = [c for c in self.checks if runner_filter.should_run_check(c)]
        with concurrent.futures.ThreadPoolExecutor() as executor:
            concurrent.futures.wait(
                [executor.submit(self.run_check_parallel, check, check_results, graph_connector)
                 for check in checks_to_run]
            )
        return check_results

    def run_check_parallel(self, check, check_results, graph_connector):
        logging.debug(f'Running graph check: {check.id}')
        passed, failed = check.run(graph_connector)
        evaluated_keys = check.get_evaluated_keys()
        check_result = self._process_check_result(passed, [], CheckResult.PASSED, evaluated_keys)
        check_result = self._process_check_result(failed, check_result, CheckResult.FAILED, evaluated_keys)
        check_results[check] = check_result

    @staticmethod
    def _process_check_result(results, processed_results, result, evaluated_keys) -> List[Dict[str, Any]]:
        for vertex in results:
            processed_results.append({"result": result, "entity": vertex, "evaluated_keys": evaluated_keys})
        return processed_results



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class VPCEndpointAcceptanceConfigured(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that VPC Endpoint Service is configured for Manual Acceptance"
        id = "CKV_AWS_123"
        supported_resources = ['AWS::EC2::VPCEndpointService']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'Properties/AcceptanceRequired'


check = VPCEndpointAcceptanceConfigured()



import logging
import os

import requests

from checkov.common.runners.base_runner import strtobool
from checkov.common.vcs.base_vcs_dal import BaseVCSDAL


class Bitbucket(BaseVCSDAL):
    def __init__(self):
        super().__init__()

    def setup_conf_dir(self):
        """
            discover parameters from execution context of checkov and determine the directory to save temporal files of vcs configuration
        """
        bitbucket_conf_dir_name = os.getenv('CKV_BITBUCKET_CONF_DIR_NAME', 'bitbucket_conf')
        self.bitbucket_conf_dir_path = os.path.join(os.getcwd(), bitbucket_conf_dir_name)
        self.bitbucket_branch_restrictions_file_path = os.path.join(self.bitbucket_conf_dir_path,
                                                                    "branch_restrictions.json")

    def discover(self):
        """
            discover parameters from execution context of checkov. usually from env variable
        """
        server_host = os.getenv('CI_SERVER_URL', "http://api.bitbucket.org/")
        self.api_url = f'{server_host}2.0'
        self.graphql_api_url = f"{server_host}api/graphql"

        self.token = os.getenv('APP_PASSWORD', '')

        self.current_repository = os.getenv('BITBUCKET_REPO_FULL_NAME', '')
        self.current_branch = os.getenv('BITBUCKET_BRANCH', '')
        self.default_branch_cache = {}
        self.username = "bschostergoi"

    def _request(self, endpoint: str):
        if not self.token:
            return
        url_endpoint = f"{self.api_url}/{endpoint}"
        try:
            s = requests.Session()
            s.auth = (self.username, self.token)
            request = s.get(url_endpoint)
            if request.status_code == 200:
                data = request.json()
                if isinstance(data, dict) and 'errors' in data.keys():
                    return None
                return data
        except Exception:
            logging.debug(f"Query failed to run by returning code of {url_endpoint}", exc_info=True)

    def get_branch_restrictions(self):
        if self.current_repository:
            branch_restrictions = self._request(
                endpoint=f"repositories/{self.current_repository}/branch-restrictions")
            return branch_restrictions
        return None

    def persist_branch_restrictions(self):
        branch_restrictions = self.get_branch_restrictions()

        if branch_restrictions:
            BaseVCSDAL.persist(path=self.bitbucket_branch_restrictions_file_path, conf=branch_restrictions)

    def persist_all_confs(self):
        if strtobool(os.getenv("CKV_BITBUCKET_CONFIG_FETCH_DATA", "True")):
            self.persist_branch_restrictions()



from typing import Any

from checkov.bicep.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class SQLServerAuditingEnabled(BaseResourceValueCheck):
    # this should be a graph check, due to the possible connection between
    # Microsoft.Sql/servers -> Microsoft.Sql/servers/auditingSettings
    # Microsoft.Sql/servers -> Microsoft.Sql/servers/databases/auditingSettings

    def __init__(self) -> None:
        name = "Ensure that 'Auditing' is set to 'Enabled' for SQL servers"
        id = "CKV_AZURE_23"
        supported_resources = (
            "Microsoft.Sql/servers/auditingSettings",
            "Microsoft.Sql/servers/databases/auditingSettings",
        )
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/state"

    def get_expected_value(self) -> Any:
        return "Enabled"


check = SQLServerAuditingEnabled()
